# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 入金伝票
class ReceivingSlipsController < ApplicationController
  before_filter :login_required
  before_filter :role_keiri_required

  verify :method => :post,
         :only => [ :destroy, :create, :update, :sign_off, :cancel_slip, 
                    :destroy_receiving ],
         :redirect_to => { :action => :list }


  # 入金・出金の両方をリスト表示
  def list
    @company = Company.find(params[:company])
    @our_bank = OurBank.find(params[:our_bank]) if !params[:our_bank].blank?
    
    if @our_bank
      @receiving_slips = ReceivingSlip.paginate :page => params[:page], 
                :per_page => 25,
                :conditions => ['our_bank_id = ?', @our_bank.id],
                :order => 'date DESC, id'
      @payment_slips = PaymentSlip.paginate :page => params[:page],
                :per_page => 25,
                :joins => "LEFT JOIN payment_banks ON payment_slips.id = payment_banks.payment_slip_id",
                :conditions => ["our_bank_id = ?", @our_bank.id],
                :order => "date DESC, id"
    else
      @receiving_slips = ReceivingSlip.paginate :page => params[:page], 
              :per_page => 25,
              :select => 'receiving_slips.*',
              :joins => 'LEFT JOIN our_banks ON receiving_slips.our_bank_id = our_banks.id',
              :conditions => ['our_banks.company_setting_id = ?', @company.id],
              :order => 'date DESC, id'
      @payment_slips = PaymentSlip.paginate :page => params[:page],
                :per_page => 25,
                :conditions => ["company_id = ?", @company.id],
                :order => "date DESC, id"
    end
  end


  # 伝票を表示
  def show
    @receiving_slip = ReceivingSlip.find params[:id]
  end


  # 新しい入金伝票
  def new
    @company = Company.find params[:company]
    @receiving_slip = ReceivingSlip.new

    @our_banks = OurBank.find :all,
                     :joins => "LEFT JOIN account_titles ON our_banks.account_title_id = account_titles.id",
                     :conditions => ["account_titles.end_day IS NULL AND company_setting_id = ?", @company.id ],
                     :order => "account_titles.account_code, account_titles.suppl_code"
    if @our_banks.empty?
      flash[:alert] = "自社銀行口座がありません。まずマスタ設定してください。"
      redirect_to :controller => "our_banks", :action => "index"
    end
  end


  # GET /receiving_slips/1/edit
  def edit
    @receiving_slip = ReceivingSlip.find(params[:id])
    @our_bank = @receiving_slip.our_bank
  end


  # 伝票の作成
  def create
    @receiving_slip = ReceivingSlip.new params[:receiving_slip]
    @receiving_slip.attributes = {
      :created_by => current_user.id,
      :state => 0,
      :amount => 0
    }
    @our_bank = @receiving_slip.our_bank

    if @receiving_slip.save
      flash[:notice] = 'ReceivingSlip was successfully created.'
      redirect_to :action => 'show', :id => @receiving_slip
    else
      render :action => "new"
    end
  end


  # PUT /receiving_slips/1
  # PUT /receiving_slips/1.xml
  def update
    @receiving_slip = ReceivingSlip.find params[:id]
    @receiving_slip.attributes = params[:receiving_slip]

    begin
      ReceivingSlip.transaction do 
        r = Receiving.find :first, :conditions => ["receiving_slip_id = ?", @receiving_slip.id]
        if r && r.sales_invoice.currency == @receiving_slip.our_bank.currency
          recalc_total_amount
        end

        @receiving_slip.save!
      end
    rescue ActiveRecord::RecordInvalid
      render :action => "edit"
      return
    end

    flash[:notice] = 'ReceivingSlip was successfully updated.'
    redirect_to :action => 'show', :id => @receiving_slip
  end


  # 伝票削除
  def destroy
    @receiving_slip = ReceivingSlip.find params[:id]
    company = @receiving_slip.our_bank.company_setting

    ReceivingSlip.transaction do 
      Receiving.delete_all ["receiving_slip_id = ?", @receiving_slip.id]
      RsLog.delete_all ["receiving_slip_id = ?", @receiving_slip.id]
      @receiving_slip.destroy
    end

    flash[:notice] = "入金伝票を削除しました。"
    redirect_to :action => 'list', :company => company
  end


  # 1行削除
  def destroy_receiving
    @receiving_slip = ReceivingSlip.find params[:id]
    receiving = Receiving.find params[:receiving]
    
    # @receiving_slip.receivings.delete @receiving
    Receiving.transaction do 
      if receiving.sales_invoice.currency.code == 
                                       @receiving_slip.our_bank.currency.code
        @receiving_slip.amount -= receiving.amount
        @receiving_slip.save!
        receiving.destroy
      else
        receiving.destroy
      end
    end

    flash[:notice] = "入金行を削除しました。"
    redirect_to :action => 'show', :id => @receiving_slip
  end


  # 経理承認
  def sign_off
    @receiving_slip = ReceivingSlip.find params[:id]
    ReceivingSlip.transaction do 
      if @receiving_slip.state < ReceivingSlip::SIGNED_OFF
        log = RsLog.new :receiving_slip => @receiving_slip,
                        :kind => RsLog::SIGN_OFF,
                        :remarks => "",
                        :done_by => current_user,
                        :done_at => Time.now
        log.save!
        @receiving_slip.state = ReceivingSlip::SIGNED_OFF
        @receiving_slip.save!
      end
    end # transaction      
    flash[:notice] = "入金伝票を承認しました。"
    redirect_to :action => "show", :id => @receiving_slip
  end


  # 未入金一覧 (表示)
  def remain
    today = Date.today
    @receiving_slip = ReceivingSlip.find params[:id]
    @currency = Currency.find params[:currency]

    @company = @receiving_slip.our_bank.company_setting
    @due_day = params['due'] ? date_from_select(params['due'], 'day') : today + 5

    @invoices = SalesInvoice.find :all,
                   :conditions => [<<EOF, @due_day, @company.id, @currency.id],
(state = 10 OR state = 20) AND (due_day <= ?) AND company_setting_id = ? AND
    currency_id = ? AND
amount_total <> (SELECT coalesce(sum(receivings.amount), 0) FROM receivings 
                    LEFT JOIN receiving_slips ON receivings.receiving_slip_id = receiving_slips.id
                    WHERE receiving_slips.state <> 30 AND sales_invoices.id = receivings.sales_invoice_id)
EOF
                     :order => 'partner_id, sell_day'
  end

  
  # 入金消し込み (実行)
  def add_receivings
    @receiving_slip = ReceivingSlip.find params[:id]
    our_bank = @receiving_slip.our_bank

    currency = nil
    # amount_total = 0
    received = []
    params[:inv].each {|key, r|
      if !r[:amount].blank? || !r[:charge].blank?
        currency = SalesInvoice.find(key).currency if !currency

        amount = nominal_amount(r[:amount], currency.exponent)
        # amount_total += amount
        received << Receiving.new(
                        :receiving_slip_id => @receiving_slip.id,
                        :amount => amount,
                        :charge => (r[:charge].blank? ? 0 : r[:charge]),
                        :sales_invoice_id => key
                    )
      end
    }
    
    ReceivingSlip.transaction do
      received.each {|r|
        r.save!
      }

      # 追加した行だけを加算するのは不味い。全部計算しなおす
      if currency == @receiving_slip.our_bank.currency
        recalc_total_amount
      end
      @receiving_slip.save!
    end # transaction

    flash[:notice] = 'Receiving was successfully created.'
    redirect_to :action => 'show', :id => @receiving_slip
  end


  def recalc_total_amount
    a = Receiving.find :first, :select => "sum(amount) as amount", 
                  :conditions => ["receiving_slip_id = ?", @receiving_slip.id]
    @receiving_slip.amount = a.amount
  end
  private :recalc_total_amount


  # 未払いの債務一覧
  def remain_payables
    @receiving_slip = ReceivingSlip.find params[:id]

    # 自分の銀行は円口座でも相手口座が外貨はありえる。
    @invoices = Invoice.find :all,
         :conditions => [],
         :order => "partner_id, buy_day"
  end


  # 印刷用画面
  def print
    @receiving_slip = ReceivingSlip.find(params[:id])
    @our_bank = @receiving_slip.our_bank
    raise TypeError if !@our_bank.is_a?(OurBank) # DEBUG
  end


  # 入金額を更新
  def update_amount
    @receiving_slip = ReceivingSlip.find params[:id]
    @receiving_slip.amount = 
            nominal_amount(params[:receiving_slip][:amount], 
                           @receiving_slip.our_bank.currency.exponent)
    @receiving_slip.save!

    flash[:notice] = 'Receiving was successfully updated.'
    redirect_to :action => 'show', :id => @receiving_slip
  end


  def cancel_slip
    receiving_slip = ReceivingSlip.find params[:id]
    company = receiving_slip.our_bank.company_setting

    ReceivingSlip.transaction do
      receiving_slip.state = 30
      receiving_slip.save!

      log = RsLog.new :receiving_slip => receiving_slip,
                      :kind => RsLog::CANCEL,
                      :remarks => "",
                      :done_by => current_user,
                      :done_at => Time.now
      log.save!
    end # of transaction
    flash[:notice] = "入金伝票をキャンセルしました。"
    redirect_to :action => "list", :company => company
  end
end
